package;

import flixel.FlxG;
import flixel.FlxState;
import flixel.system.FlxSound;
import flixel.tweens.FlxTween;
import openfl.Assets;
import states.IntroState;
import states.CutsceneState;
import states.PlayState;

import event.*;

typedef BGM = { snd:FlxSound, vol:Float }
typedef SFX = { name:String, vol:Float }

class AudioMixer
{
	static private var SUFFIX:String = ".mp3";

	private static var bgmArray:Array<BGM>;
	static private var bgmInstance:FlxSound;

	private static var ambienceMain:FlxSound;

	private static var heartbeatSound:FlxSound;
	private static var tickingSound:FlxSound;

	// For automatic registration and playing of sound effects
	static private var autoSfxMap:Map<Int, SFX> = [
		EventType.DISPLAY_INFO  => { name: SND("sounds/Opening patient's file-trimmed"), vol: 0.7 },
		EventType.TREAT_PATIENT  => { name: SND("sounds/Button click"), vol: 0.8 },
		EventType.CLEAR_PATIENT  => { name: SND("sounds/Button click"), vol: 0.8 },
		EventType.DISPLAY_CANCEL => { name: SND("sounds/Button click"), vol: 0.8 },
	];

	private static var inited:Bool = false;

	public static function init():Void
	{
		// Create the background loops
		bgmArray = [
			{ snd: FlxG.sound.load(SND("music/GGJ 2015 intro"), 1, true, false, false), vol: 0.45 },
			{ snd: FlxG.sound.load(SND("music/GGJ 2015 main"), 1, true, false, false), vol: 0.45 },
		];
		for (bgm in bgmArray)
		{
			bgm.snd.survive = true;
		}

		ambienceMain = FlxG.sound.load(SND("sounds/Hospital ambience - After the attack starts"), 1, true, false, false);
		ambienceMain.survive = true;

		inited = true;
	}

	public static function enterState(state:FlxState):Void
	{
		if (!inited)
			init();

		var nextSound:BGM = null;
		if (Std.is(state, IntroState))
		{
			nextSound = bgmArray[0];
		}
		else if (Std.is(state, CutsceneState))
		{
			stopBgm();
			ambienceMain.play();
			ambienceMain.fadeIn(0.1, 0, 0.25);
		}
		else if (Std.is(state, PlayState))
		{
			nextSound = bgmArray[1];

			// Need to re-subscribe to events
			//EventManager.subscribe(EventType.PLAYER_STEP, onPlayerStep);
		}

		// Automagically register the events in the autoSfxMap
		for (evtName in autoSfxMap.keys())
			EventManager.subscribe(evtName, playSfx);

		if (nextSound != null)
			swapBGM(nextSound.snd, nextSound.vol);
	}

	public static function stopBgm(eventType:Int=0, ?params:Dynamic):Void
	{
		if (bgmInstance != null)
		{
			bgmInstance.fadeOut(0.5, 0);
			bgmInstance = null;
		}
	}

	private static function swapBGM(sound:FlxSound, vol:Float, fadeDur:Float=1):Void
	{
		if (bgmInstance != null && bgmInstance != sound)
		{
			bgmInstance.fadeOut(fadeDur, 0);
		}

		bgmInstance = sound;
		bgmInstance.play(true);
		bgmInstance.fadeIn(fadeDur, 0, vol);
	}

	public static function playHeartbeat(state:String, volFactor:Float=1):Void
	{
		var sfxname:String = "";
		switch (state)
		{
			case "ill/injured":
				sfxname = SND("sounds/Heartbeat monitor normal");
			case "urgent":
				sfxname = SND("sounds/Heartbeat monitor urgent");
			case "critical":
				sfxname = SND("sounds/Heartbeat monitor critical");
			case "deceased":
				sfxname = SND("sounds/Heartbeat monitor dead");
		}
		if (sfxname != "")
		{
			stopHeartbeat();
			heartbeatSound = FlxG.sound.play(sfxname, 1.0*volFactor, true);
			heartbeatSound.fadeIn(0.2, 0, 1.0*volFactor);
		}
	}
	public static function playHeartbeatLouder(state:String):Void
	{
		// Assume that we want to make the existing heartbeat louder
		if (heartbeatSound != null)
		{
			heartbeatSound.fadeIn(0.2, heartbeatSound.volume, 1.0);
		}
		else
		{
			playHeartbeat(state);
		}
	}
	public static function stopHeartbeat():Void
	{
		if (heartbeatSound != null)
		{
			fadeOutAndKill(heartbeatSound, 0.5, 0);
			heartbeatSound = null;
		}
	}

	public static function startTicking():Void
	{
		if (tickingSound == null)
			tickingSound = FlxG.sound.play(SND("sounds/ticking-clock"), 0.5, true);
	}
	public static function stopTicking():Void
	{
		if (tickingSound != null)
		{
			fadeOutAndKill(tickingSound, 0.8, 0);
			tickingSound = null;
		}
	}

	public static function playSfx(eventType:Int=0, ?params:Dynamic):Void
	{
		if (autoSfxMap.exists(eventType))
		{
			var sfx:SFX = autoSfxMap.get(eventType);
			FlxG.sound.play(sfx.name, sfx.vol, false, true);
		}
	}

	public static function muteAll(val:Bool):Void
	{
		FlxG.sound.muted = val;
	}

	public static function update():Void
	{
	}

	// Helper function to append the correct extension to sound file names
	private static inline function SND(name:String):String
	{
		return name + SUFFIX;
	}

	private static function fadeOutAndKill(snd:FlxSound, Duration:Float, ?To:Float=0):Void
	{
		FlxTween.singleVar(snd, "volume", To, Duration, { complete: function(tween:FlxTween):Void { snd.stop(); } });
	}
}
