package clock;
import event.EventManager;
import event.EventType;
import flixel.FlxG;
import flixel.group.FlxGroup;
import flixel.text.FlxText;

/**
 * ...
 * @author chinykian
 */
class TimeDisplay extends FlxGroup
{
	private static var TURN_INTERVAL:Int = 10;
	private static var TEXT_SIZE:Int = 16;
	
	//members
	private var mElapsed:Float = 0;
	private var mScaleMod:Float = 1;
	
	private var mHour:FlxText;
	private var mMin:FlxText;
	private var mAM:FlxText;
	
	private var mHourPart:Int;
	private var mMinPart:Int;
	
	private var mAdditionalMsg:FlxText;
	
	private var mIsAM:Bool = true;

	public function new()
	{
		super();
		
		mHourPart = 12;
		mMinPart = 0;
		
		mHour = new FlxText(FlxG.width * 0.5 - 90, FlxG.height - 30, 100, "12:", TEXT_SIZE);
		mHour.alignment = "right";
		mMin = new FlxText(mHour.x + mHour.width - 4, mHour.y, 50, "00", TEXT_SIZE);
		mMin.alignment = "left";
		
		mAM = new FlxText(mMin.x + mMin.width - 22, mHour.y + 7, 100, "a.m.", 8);
		
		add(mHour);
		add(mMin);
		add(mAM);
		
		mAdditionalMsg = new FlxText(0, 0, FlxG.width, "10 MINUTES PASSED");
		mAdditionalMsg.alignment = "center";
		mAdditionalMsg.color = 0xffffff00;
		mAdditionalMsg.visible = false;
		add(mAdditionalMsg);
	}
	
	public function initEntity(_parent:FlxGroup):Void
	{
		_parent.add(this);
		
		EventManager.subscribe(EventType.TURN_PASSED_OVER, onTurnPassOver);
	}
	
	private function onTurnPassOver(evt:Int, params:Dynamic):Void
	{
		mScaleMod = 1.5;
		mHour.color = 0xffffff00;
		mMin.color = 0xffffff00;
		
		mHour.alpha = 1;
		mMin.alpha = 1;
		mAM.alpha = 1;
		
		mAdditionalMsg.y = FlxG.height - 40;
		mAdditionalMsg.alpha = 1;
		mAdditionalMsg.visible = true;
		
		mMinPart += TURN_INTERVAL;
		if (mHourPart == 12 && mMinPart == 0)
			mIsAM = !mIsAM;
		if (mMinPart >= 60)
		{
			mMinPart -= 60;
			mHourPart++;
			
			if ((mHourPart % 13) == 0 && !(mHourPart == 0 && mMinPart == 0))
				mHourPart -= 12;
		}
		
		mHour.text = toDoubleDigitString(mHourPart) + ":";
		mMin.text = toDoubleDigitString(mMinPart);
		
		if (!mIsAM)
			mAM.text = "p.m.";
		else
			mAM.text = "a.m.";
	}
	
	private function toDoubleDigitString(val:Int):String
	{
		if (val < 10)
			return "0" + val;
		return val + "";
	}
	
	public override function update():Void
	{
		mScaleMod -= FlxG.elapsed * 2;
		mScaleMod = Math.max(mScaleMod, 1);
		
		mAdditionalMsg.alpha -= FlxG.elapsed;
		mAdditionalMsg.y -= FlxG.elapsed * 12;
		mAdditionalMsg.alpha = Math.max(mAdditionalMsg.alpha, 0);
		
		mElapsed += FlxG.elapsed * 4;
		var alphaVal:Float = (Math.sin(mElapsed) * 0.5 + 0.5) * 0.5 + 0.5;
		mHour.alpha = alphaVal;
		mMin.alpha = alphaVal;
		mAM.alpha = alphaVal;
		
		mHour.size = mScaleMod * TEXT_SIZE;
		mMin.size = mScaleMod * TEXT_SIZE;
		
		// reset color
		if (mScaleMod == 1) //assignment confirmed
		{
			mHour.color = 0xffffffff;
			mMin.color = 0xffffffff;
		}
		
		super.update();
	}
	
}