package fsm;

/**
 * ...
 * @author chinykian
 */
class FSM
{
	
	private var mStates:Map < String, Array < Void->Void > > ;
	private var mEdges:Map < String, Map < String, Void->Bool > > ; //current statename, target statename+edge predicate
	private var mEdgesKeyCache:Map<String, Array<String>>; // cache of all the keys in the edges map
	private var mCurrentState:String = "";
	private var mFirstStateName:String;

	public function new()
	{
		mStates = new Map < String, Array < Void->Void > > ();
		mEdges = new Map < String, Map < String, Void->Bool > > ();
		mEdgesKeyCache = new Map<String, Array<String>>();
	}
	
	public function getState():String
	{
		return mCurrentState;
	}
	
	/**
	 * Takes in an array of functions: enter, loop, exit
	 * @param	stateFuncs : array of functions: enter, loop, exit
	 * @param	edges : targetState => predicate
	 */
	public function addState(name:String, stateFuncs:Array<Void->Void>, edges:Map < String, Void->Bool >):Void
	{
		mStates.set(name, stateFuncs);
		mEdges.set(name, edges);

		// Cache the keys of the edges
		var keys = new Array<String>();
		if (edges != null)
		{
			for (key in edges.keys())
				keys.push(key);
		}
		mEdgesKeyCache.set(name, keys);

		if (mCurrentState == "")
		{
			mCurrentState = name;
			mFirstStateName = name;
			mStates[mCurrentState][0]();
		}
	}
	
	public function reset():Void
	{
		mCurrentState = mFirstStateName;
		mStates[mCurrentState][0]();
	}
	
	public function update():Void
	{
		if (mCurrentState == "")
			return;
		mStates[mCurrentState][1]();
		var edges = mEdges[mCurrentState];
		if (edges != null)
		{
			for ( target in mEdgesKeyCache[mCurrentState] )
			{
				if (edges[target]())
				{
					changeState(target);
					break;
				}
			}
		}
	}
	
	public function changeState(stateName:String):Void
	{
		if (mStates.exists(stateName) == true)
		{
			mStates[mCurrentState][2](); // exit()
			mCurrentState = stateName;
			mStates[mCurrentState][0](); // enter()
		}
	}
	
}
