package hud;

import event.EventManager;
import event.EventType;
import flixel.FlxG;
import flixel.FlxSprite;
import flixel.group.FlxGroup;
import flixel.group.FlxSpriteGroup;
import flixel.text.FlxText.FlxTextFormat;
import flixel.util.FlxPoint;
import flixel.util.FlxSpriteUtil;
import patients.Patient;
import utils.FlxTextExt;

class MiniInfoDisplay extends FlxSpriteGroup
{
	private var parent:FlxGroup;

	private var mCondition:FlxTextExt;
	private var mTreatmentTime:FlxTextExt;
	private var mBG:FlxSprite;

	private var mCurrentFormat:FlxTextFormat;

	private static inline var BG_COLOR:Int = 0xff000000;
	private static inline var LINE_SPACE:Int = 5;
	private static var TEXT_OFFSET:FlxPoint = new FlxPoint(4, 4);

	private var TEXT_FORMAT = [
		"ill/injured" => new FlxTextFormat(0xff67a416, false, false, null, 10, 100),
		"urgent"      => new FlxTextFormat(0xffffb144, false, false, null, 10, 100),
		"critical"    => new FlxTextFormat(0xffba3d1b, false, false, null, 10, 100),
		"deceased"    => new FlxTextFormat(0xff666666, false, false, null, 10, 100),
	];

	public function new()
	{
		super();
	}

	public override function get_width():Float
	{
		return mBG.width;
	}

	public function initEntity(_parent:FlxGroup):Void
	{
		parent = _parent;
		parent.add(this);

//		EventManager.subscribe(EventType.MESSAGE_DISPLAY, onMessageDisplay);
//		EventManager.subscribe(EventType.MESSAGE_HIDE, onMessageHide);

		mBG = new FlxSprite();
		mBG.makeGraphic(1, 1, 0x0);
		add(mBG);

		mCondition = new FlxTextExt(0, 0, 100);
		mCondition.wordWrap = true;
		mCondition.width = FlxG.width - 8;
		mCondition.text = "TEMP\nTEMP";
		mCondition.x = TEXT_OFFSET.x;
		mCondition.y = TEXT_OFFSET.y;
		add(mCondition);

		mTreatmentTime = new FlxTextExt(0, 0, 100);
		mTreatmentTime.wordWrap = true;
		mTreatmentTime.width = FlxG.width - 8;
		mTreatmentTime.text = "TEMP\nTEMP";
		mTreatmentTime.x = TEXT_OFFSET.x;
		mTreatmentTime.y = mCondition.y + mCondition.getTextHeight() + LINE_SPACE;
		add(mTreatmentTime);

		x = FlxG.width + 100; //HACK: put this off the screen
	}

	public function showInfo(patient:Patient, origin:FlxPoint):Void
	{
		x = 0;
		y = 0;

		//TODO: animate the display

		var textWidth:Int = 0;
		var textHeight:Int = 0;

		mCondition.text = "Condition\n" + patient.condition.toUpperCase();
		if (patient.condition == "deceased")
		{
			mTreatmentTime.text = "Treatment\n--";
		}
		else
		{
			mTreatmentTime.text = "Treatment\n" + PatientInfoDisplay.getTreatmentTimeString(patient.treatmentTime);
		}
		textWidth = Std.int(Math.max(mCondition.getTextWidth(), mTreatmentTime.getTextWidth()));
		textHeight += Std.int(mCondition.getTextHeight()) + LINE_SPACE;
		textHeight += Std.int(mTreatmentTime.getTextHeight());

		var fullWidth:Int = textWidth + 20;
		var fullHeight:Int = textHeight + 20;

		mBG.makeGraphic(fullWidth, fullHeight, 0x0);
		FlxSpriteUtil.drawRect(mBG, 0, 0, mBG.width, mBG.height, BG_COLOR);

		//NOTE: can't use FlxText.clearFormats because it disposes the FlxTextFormat
		if (mCurrentFormat != null)
			mCondition.removeFormat(mCurrentFormat);
		mCurrentFormat = TEXT_FORMAT[patient.condition];
		mCondition.addFormat(mCurrentFormat);

		x = origin.x;
		y = origin.y;
	}

	public function hide()
	{
		//HACK: just position off-screen
		x = FlxG.width + 100;
	}
}
