package hud;

import event.EventManager;
import event.EventType;
import flixel.FlxG;
import flixel.FlxSprite;
import flixel.ui.FlxButton;
import flixel.group.FlxGroup;
import flixel.group.FlxSpriteGroup;
import flixel.text.FlxText.FlxTextFormat;
import flixel.util.FlxPoint;
import flixel.util.FlxSpriteUtil;
import objects.GameObject;
import openfl.Assets;
import patients.Patient;
import tweenx909.EaseX;
import tweenx909.TweenX;
import utils.FlxTextExt;

class PatientInfoDisplay extends FlxSpriteGroup
{
	private var parent:FlxGroup;

	private var mName:FlxTextExt;
	private var mCondition:FlxTextExt;
	private var mAge:FlxTextExt;
	private var mTreatmentTime:FlxTextExt;
	private var mDialogue:FlxTextExt;
	private var mProfileImg:FlxSprite;
	private var mProfileImgBg:FlxSprite;
	private var mBG:FlxSprite;

	private var mCurrentFormat:FlxTextFormat;

	private var mTreatBtn:FlxButton;
	private var mCancelBtn:FlxButton;

	private static inline var BG_COLOR:Int = 0xff000000;
	private static inline var RIGHT_MARGIN:Int = 8;

	private static inline var LINE_SPACE:Int = 5;
	private static var TEXT_OFFSET:FlxPoint = new FlxPoint(105, 10);
	private static var COLUMN2_X:Int = 150;

	private var TEXT_FORMAT = [
		"ill/injured" => new FlxTextFormat(0xff67a416, false, false, null, 11, 100),
		"urgent"      => new FlxTextFormat(0xffffb144, false, false, null, 11, 100),
		"critical"    => new FlxTextFormat(0xffba3d1b, false, false, null, 11, 100),
		"deceased"    => new FlxTextFormat(0xff666666, false, false, null, 11, 100),
	];

	public function new()
	{
		super();
	}

	public override function get_width():Float
	{
		return mBG.width;
	}

	public function initEntity(_parent:FlxGroup):Void
	{
		parent = _parent;
		parent.add(this);

//		EventManager.subscribe(EventType.MESSAGE_DISPLAY, onMessageDisplay);
//		EventManager.subscribe(EventType.MESSAGE_HIDE, onMessageHide);

		mBG = new FlxSprite();
		mBG.makeGraphic(1, 1, 0x0);
		add(mBG);

		mProfileImgBg = new FlxSprite();
		mProfileImgBg.makeGraphic(64, 64, 0xffaaaaaa);
		mProfileImgBg.x = 10;
		mProfileImgBg.y = 10;
		add(mProfileImgBg);

		mProfileImg = new FlxSprite();
		mProfileImg.makeGraphic(64, 64, 0xff864523);
		mProfileImg.x = 10;
		mProfileImg.y = 10;
		add(mProfileImg);

		mName = new FlxTextExt(0, 0, 100);
		mName.width = FlxG.width - 8 - RIGHT_MARGIN;
		mName.text = "TEMP";
		mName.x = TEXT_OFFSET.x;
		mName.y = TEXT_OFFSET.y;
		add(mName);

		mAge = new FlxTextExt(0, 0, 100);
		mAge.width = FlxG.width - 8 - RIGHT_MARGIN;
		mAge.text = "TEMP";
		mAge.x = TEXT_OFFSET.x + COLUMN2_X;
		mAge.y = mName.y;
		add(mAge);

		mCondition = new FlxTextExt(0, 0, 100);
		mCondition.width = FlxG.width - 8 - RIGHT_MARGIN;
		mCondition.text = "TEMP";
		mCondition.x = TEXT_OFFSET.x;
		mCondition.y = mName.y + mName.getTextHeight() + LINE_SPACE;
		add(mCondition);

		mTreatmentTime = new FlxTextExt(0, 0, 100);
		mTreatmentTime.width = FlxG.width - 8 - RIGHT_MARGIN;
		mTreatmentTime.text = "TEMP";
		mTreatmentTime.x = TEXT_OFFSET.x + COLUMN2_X;
		mTreatmentTime.y = mCondition.y;
		add(mTreatmentTime);


		mTreatBtn = new FlxButton(TEXT_OFFSET.x, mCondition.y + mCondition.getTextHeight() + LINE_SPACE*2, "TREAT", onTreat);
		add(mTreatBtn);

		mCancelBtn = new FlxButton(TEXT_OFFSET.x + 100, mCondition.y + mCondition.getTextHeight() + LINE_SPACE*2, "CANCEL", onCancel);
		add(mCancelBtn);

		mDialogue = new FlxTextExt(0, 0, 100);
		mDialogue.width = FlxG.width - 8 - RIGHT_MARGIN;
		mDialogue.wordWrap = true;
		mDialogue.text = "QUOTE";
		mDialogue.x = 10;
		add(mDialogue);


		x = FlxG.width + 100; //HACK: put this off the screen
	}

	public function showInfo(patient:Patient, origin:FlxPoint):Void
	{
		x = 0;
		y = 0;

		//TODO: animate the display

		var textWidth = 0;
		var textHeight = 0;

		mName.text = "Name: " + patient.name;
		mAge.text = "Age: " + patient.age;
		textWidth = Std.int(Math.max(textWidth, COLUMN2_X + mAge.getTextWidth()));
		textHeight += Std.int(mName.getTextHeight());

		mCondition.text = "Condition: " + patient.condition.toUpperCase();
		if (patient.condition == "deceased")
		{
			mDialogue.text = "";
			mTreatmentTime.text = "Treatment: --";
			mTreatBtn.label.text = "CLEAR";
		}
		else
		{
			mTreatmentTime.text = "Treatment: " + getTreatmentTimeString(patient.treatmentTime);
			mTreatBtn.label.text = "TREAT";
			mDialogue.text = "\"" + patient.quote + "\"";
		}
		textWidth = Std.int(Math.max(textWidth, COLUMN2_X + mTreatmentTime.getTextWidth()));
		textHeight += Std.int(mCondition.getTextHeight()) + LINE_SPACE;

		//textWidth = Std.int(Math.max(textWidth, mCancelBtn.x));
		textHeight += Std.int(mCancelBtn.height) + LINE_SPACE;

		var fullWidth:Int = Std.int(textWidth+20+TEXT_OFFSET.x);
		var fullHeight:Int = Std.int(Math.max(textHeight, mProfileImg.height)) + 20;

		mDialogue.width = fullWidth-20;
		mDialogue.y = fullHeight;

		fullHeight += Std.int(mDialogue.height + 20);

		mBG.makeGraphic(fullWidth, fullHeight, 0x0);
		FlxSpriteUtil.drawRect(mBG, 0, 0, mBG.width, mBG.height, BG_COLOR);

		mProfileImg.loadGraphic(Assets.getBitmapData("assets/img/" + patient.imagePath));

		//NOTE: can't use FlxText.clearFormats because it disposes the FlxTextFormat
		if (mCurrentFormat != null)
			mCondition.removeFormat(mCurrentFormat);
		mCurrentFormat = TEXT_FORMAT[patient.condition];
		mCondition.addFormat(mCurrentFormat);

	}
	
	/*
	public function showInfo(info:Info, origin:FlxPoint):Void
	{
		var tweenIn = function():Void {
			mText.text = info;
			mTextWidth = Std.int(mText.getTextWidth());
			mBG.makeGraphic(mTextWidth+15, 21, 0x0);
			FlxSpriteUtil.drawCircle(mBG, 5, 9, 6, BG_COLOR);
			FlxSpriteUtil.drawCircle(mBG, mBG.width-5, 9, 6, BG_COLOR);
			FlxSpriteUtil.drawRect(mBG, 7, 4, mBG.width-14, 6*2-1, BG_COLOR);

			TweenX.to(this, { y:100 }, 0.2, EaseX.backOut, 0.25);
		}

		if (isOnScreen())
			hide().onFinish(tweenIn);
		else
			tweenIn();
	}
	*/

	public function hide()
	{
		//return TweenX.to(this, { y:100 }, 0.2, EaseX.backIn, 0.25);

		//HACK: just position off-screen
		x = FlxG.width + 100;
		EventManager.triggerEvent(EventType.DISPLAY_CANCEL, null);
	}

	public override function update():Void
	{
		super.update();
	}

	public static function getTreatmentTimeString(treatmentTurns):String
	{
		return (treatmentTurns * 10) + "min";
	}

	private function onMessageDisplay(evt:Int, params:Dynamic):Void
	{
		showInfo(params.text, new FlxPoint(0, 0));
	}
	private function onMessageHide(evt:Int, params:Dynamic):Void
	{
		hide();
	}

	private function onTreat():Void
	{
		if (mTreatBtn.label.text.toUpperCase() == "TREAT")
		{
			EventManager.triggerEvent(EventType.TREAT_PATIENT, null);
		}
		else
		{
			EventManager.triggerEvent(EventType.CLEAR_PATIENT, null);
		}
		hide();
	}

	private function onCancel():Void
	{
		hide();
	}
}
