package objects;
import event.EventManager;
import event.EventType;
import flixel.FlxG;
import flixel.FlxSprite;
import flixel.group.FlxGroup;
import flixel.util.FlxSpriteUtil;
import openfl.Assets;
import patients.Patient;
import states.PlayState;

/**
 * ...
 * @author chinykian
 */
class BedObject extends GameObject
{
	public static var allBeds:Array<BedObject>;

	//members
	private var mPatient:Patient = null;
	private var mFrontFrame:FlxSprite;
	private var mSelector:FlxSprite = null;
	private var mIndex:Int = 0; //hack
	
	private var mIsProcessing:Bool = false;

	public var patient(get,never):Patient;
	function get_patient():Patient { return mPatient; }
	
	private var mFly:Array<Fly>;

	public var onMouseOver:BedObject->Void;
	public var onMouseOut:BedObject->Void;

	public function new() 
	{
		super();
		
		loadGraphic(Assets.getBitmapData("assets/img/bed/bed_back.png"));
		mFrontFrame = new FlxSprite(0, 0, Assets.getBitmapData("assets/img/bed/bed_front.png"));
		mPatient = new Patient();
		mSelector = new FlxSprite(0, 0, Assets.getBitmapData("assets/img/bed/bed_select.png"));
		mSelector.offset.set(1, 1);
		mSelector.alpha = 0.25;

		mIndex = allBeds.length;
		allBeds.push(this);
		
		mFly = new Array<Fly>();
		mFly.push(new Fly());
		mFly.push(new Fly());
		mFly.push(new Fly());
	}
	
	/**
	 * 
	 */
	public override function initEntity(_parent:FlxGroup):Void
	{
		super.initEntity(_parent);

		mPatient.setPosition(x, y);
		addSubSprite(mPatient);

		mFrontFrame.setPosition(x, y);
		addSubSprite(mFrontFrame);
		
		mSelector.setPosition(x, y);
		addSubSprite(mSelector);
		mSelector.visible = false;
		
		for (fly in mFly)
		{
			fly.setPosition(x, y);
			fly.start(centerX, centerY);
			addSubSpriteToLayer(fly, MapLoader.displayLayers[MapLoader.PARTICLES]);
			fly.visible = false;
		}
		
		EventManager.subscribe(EventType.DISPLAY_CANCEL, onDisplayCancel);
		EventManager.subscribe(EventType.PATIENT_CURED, onPatientCured);
	}

	public override function update():Void
	{
		var playState = cast(FlxG.state, PlayState);
		if (!playState.mAreBedEnabled)
			return;
		
		if (isOccupied())
		{
			var isMouseHit = FlxG.mouse.inCoords(x, y, width, height);
			if (FlxG.mouse.justReleased && isMouseHit && !patient.isFadingOut())
			{
				FlxSpriteUtil.flicker(mSelector, 0.25);
				EventManager.triggerEvent(EventType.DISPLAY_INFO, {patient:mPatient});
				mIsProcessing = true;
			}
			if (!mIsProcessing)
			{
				if (isMouseHit)
				{
					if (!mSelector.visible)
					{
						mSelector.visible = true;
						if (onMouseOver != null)
							onMouseOver(this);
					}
				}
				else
				{
					if (mSelector.visible)
					{
						mSelector.visible = false;
						if (onMouseOut != null)
							onMouseOut(this);
					}
				}
			}
				
			setBedColor(0xffffffff);
			
		}
		else
			setBedColor(0xffcccccc);
	}
	
	private function setBedColor(_color:Int):Void
	{
		color = _color;
		mFrontFrame.color = _color;
	}
	
	public function isOccupied():Bool
	{
		return mPatient.animation.curAnim.name != "empty";
	}
	
	public function spawnPatient():Void
	{
		if (isOccupied())
			return; //already occupied
		mPatient.generateDataAndSpawn();
		clearFlies();
		
		Monitor.AllMonitors[mIndex].setStatus(mPatient.condition);
	}
	
	private function clearFlies():Void
	{
		for (fly in mFly)
			fly.visible = false;
	}

	public function updateStatus():Void
	{
		if (isOccupied())
		{
			if (mPatient.condition == "deceased")
			{
				mPatient.advanceDecayLevel();
				for (i in 0 ... mPatient.decayLevel)
					mFly[i].visible = true;
			}
			else if (ConditionProbabilities.shouldAdvance(mPatient.condition, mPatient.age))
			{
				mPatient.advanceCondition();
			}
			Monitor.AllMonitors[mIndex].setStatus(mPatient.condition);
		}
	}
	
	private function onDisplayCancel(evt:Int, params:Dynamic):Void
	{
		mIsProcessing = false;
	}
	
	private function onPatientCured(evt:Int, params:Dynamic):Void
	{
		if (params.patient == patient)
		{
			Monitor.AllMonitors[mIndex].setStatus("idle");
			clearFlies();
		}
	}
	
}

class ConditionProbabilities
{
	public static var StatusToProbability = [
		"ill/injured" => 0.5, //0.7
		"urgent" => 0.7,      //0.8
		"critical" => 0.9];   //0.9

	public static function ageGroupToProbability(age:Int):Float
	{
		if (age <= 12)
			return 0.9;
		else if (age <= 21)
			return 0.7;
		else if (age <= 35)
			return 0.5;
		else if (age <= 50)
			return 0.5;
		else if (age <= 65)
			return 0.7;
		else
			return 0.9;
	}

	public static function shouldAdvance(status:String, age:Int):Bool
	{
		var r = Math.random();
		var statusProb = StatusToProbability[status];
		var ageProb = ageGroupToProbability(age);
		var prob = statusProb * ageProb;
		trace("Rolled " + r + " / " + prob + " (" + statusProb + " * " + ageProb + ")");
		return r < prob;
	}
}
