package objects;
import flixel.FlxBasic;
import flixel.FlxSprite;
import flixel.FlxState;
import flixel.group.FlxGroup;
import flixel.util.FlxPoint;
import event.EventManager;

typedef EventCallbackPair = { event:Int, callback:Int->Dynamic->Void }

/**
 * ...
 * @author chinykian
 */
class GameObject extends FlxSprite
{
	public var initialPosition:FlxPoint;
	public var name:String; //do not modify outside of gameobjectmanager
	public var parent:FlxGroup;
	public var loadedName:String; //do not modify outside of maploader

	public var isFree:Bool = true;

	private var mEventCallbackPairs:Array<EventCallbackPair>;
	private var mSubSprites:Array<FlxBasic>;
	private var mSubSpritesForDisplayLayers:Array<Dynamic>; //layer:Maploader.displaylayer, sprite:flxsprite
	private var mCurrAnimState:String;
	
	public function new() 
	{
		super();
		initialPosition = new FlxPoint();
		mEventCallbackPairs = new Array<EventCallbackPair>();
		mSubSprites = new Array<FlxBasic>();
		mSubSpritesForDisplayLayers = new Array<Dynamic>();
		mCurrAnimState = "idle";
	}
	
	public function addToState(state:FlxState):Void
	{
	}
	
	public var centerX(get, null):Float;
	public function get_centerX():Float
	{
		return x + width * 0.5;
	}
	
	public var centerY(get, null):Float;
	public function get_centerY():Float
	{
		return y + height * 0.5;
	}
	
	/**
	 * Called in maploader
	 * @param	_parent
	 */
	public function initEntity(_parent:FlxGroup):Void
	{
		parent = _parent;
		parent.add(this);
	}

	/**
	 * Sets a property of the object by the property name via reflection.
	 * @param	_propName	property name
	 * @param	_value		value to be set
	 */
	public function setProperty(_propName:String, _value:Dynamic):Void
	{
		Reflect.setProperty(this, _propName, _value);
	}
	
	/**
	 * Called each time object is retrieved from pool
	 */
	public function onSpawn():Void
	{
	}

	/**
	 * Adds the subsprite to the parent and keeps track of it.
	 */
	public function addSubSprite(obj:FlxBasic):Void
	{
		if (parent != null)
			parent.add(obj);
		mSubSprites.push(obj);
	}
	
	public function addSubSpriteToLayer(obj:FlxBasic, displaylayer:FlxGroup):Void
	{
		mSubSpritesForDisplayLayers.push( { sprite:obj, layer:displaylayer } );
		displaylayer.add(obj);
	}

	/**
	 * Returns the object to the pool.
	 */
	public function returnToPool():Void
	{
		unsubscribe();
		if (parent != null)
			parent.remove(this);
		while (mSubSprites.length > 0)
			parent.remove(mSubSprites.pop());
		while (mSubSpritesForDisplayLayers.length > 0)
		{
			var obj:Dynamic = mSubSpritesForDisplayLayers.pop();
			obj.layer.remove(obj.sprite);
		}
		isFree = true;
	}

	/**
	 * Subscribes to every event given in the list. Should not be overridden.
	 * @param list of (event, callback) pairs
	 */
	public function subscribe(eventCallbackPairs:Array<EventCallbackPair>):Void
	{
		mEventCallbackPairs = eventCallbackPairs;
		for (pair in mEventCallbackPairs)
			EventManager.subscribe(pair.event, pair.callback);
	}

	/**
	 * Unsubscribes previous subscriptions. Should not be overridden.
	 */
	public function unsubscribe():Void
	{
		for (pair in mEventCallbackPairs)
			EventManager.unsubscribe(pair.event, pair.callback);
		mEventCallbackPairs.splice(0, mEventCallbackPairs.length);
	}

}
