package objects;
import event.EventManager;
import event.EventType;
import objects.GameObject;

/**
 * ...
 * @author chinykian
 */
class GameObjectManager
{

	private static var mObjects:Map<String, GameObject> = null;
	private static var mCount:Map < String, Int> = null; // tracks # of objects which have same name
	
	private static var mPools:Map<String, Array<GameObject>>;
	private static var mPoolIndexes:Map<String, Int>;
	
	private static var mSpawnObjects:Array<GameObject> = null;
	
	private static inline var POOL_SIZE:Int = 50;
	
	public static function init():Void
	{
		mObjects = new Map<String, GameObject>();
		mCount = new Map < String, Int>();
		mSpawnObjects = new Array<GameObject>();
		
		// setup pool
		mPoolIndexes = new Map<String, Int>();
		mPools = new Map < String, Array<GameObject> > ();
		//makePool("Player", 1);
		
		EventManager.subscribe(EventType.LEVEL_DESTROY, onDestroy);
	}
	
	private static function onDestroy(type:Int, params:Dynamic):Void
	{
		mSpawnObjects = new Array<GameObject>();
	}
	
	public static function getObjectFromPool(type:String):GameObject
	{
		// make pool if pool does not exist
		//if (mPools.exists(type) == false)
		//	makePool(type, POOL_SIZE);
		//
		//var obj = getNextAvailable(type);
		//obj.reset(-100, -100);
		//
		//obj.onSpawn();
		//EventManager.triggerEvent(EventType.OBJECT_SPAWNED, {object:obj});
		
		var obj =  createObject(type);
		obj.reset(-100, -100);
		
		obj.onSpawn();
		EventManager.triggerEvent(EventType.OBJECT_SPAWNED, {object:obj});
		
		return obj;
	}

	private static function getNextAvailable(type:String):GameObject
	{
		var pool = mPools[type];
		var index = mPoolIndexes[type];
		var len = pool.length;
		var i = index;
		while (!pool[i].isFree)
		{
			i = (i+1) % len;
			if (i == index) // Went one round back to current index
				throw "No available slots in pool type " + type;
		}
		var obj = pool[i];
		obj.isFree = false;
		mPoolIndexes[type] = (i+1) % len;
		return obj;
	}
	
	private static function makePool(type:String, size:Int):Void
	{
		mPoolIndexes.set(type, 0);
		mPools.set(type, new Array<GameObject>());
		for (i in 0 ... size)
		{
			mPools[type].push( createObject(type) );
		}
	}
	
	private static function createObject(type:String):GameObject
	{		
		var name = type;
		var classType = Type.resolveClass("objects." + type);
		if (classType == null)
			throw "Invalid object type";
		var obj = Type.createInstance(classType, []);
		if (mCount.exists(type) == true)
		{
			mCount[type] += 1;
			name += mCount[type];
		}
		else
			mCount.set(type, 1);
			
		mObjects.set(name, obj);
		obj.name = name;
		//MapLoader.displayLayers[1].add(obj);
		obj.kill();
		return obj;
	}
	
}
