package objects;
import event.EventManager;
import event.EventType;
import flixel.addons.tile.FlxTilemapExt;
import flixel.FlxG;
import flixel.FlxSprite;
import flixel.group.FlxGroup;
import flixel.text.FlxText;
import flixel.tile.FlxTilemap;
import flixel.util.FlxStringUtil;
import haxe.Json;
import openfl.Assets;

/**
 * ...
 * @author chinykian
 */
class MapLoader
{
	public static var TILE_SIZE:Int = 32; //Warning: don't change this value from outside this class!
	
	private static var mCollisionMap:FlxTilemapExt;
	public static var currentMap(get,never):FlxTilemapExt;
	static function get_currentMap() { return mCollisionMap; }
	
	public static inline var BACKGROUND:Int   = 0;
	public static inline var TERRAIN:Int      = 1;
	public static inline var OBJECTLAYER0:Int = 2;
	public static inline var OBJECTLAYER1:Int = 3;
	public static inline var OBJECTLAYER2:Int = 4;
	public static inline var PARTICLES:Int    = 5;
	public static inline var ITEMS:Int        = 6;
	public static inline var FADINGLAYER:Int  = 7; //for fading the black background in
	public static inline var OVERLAY:Int      = 8;
	public static inline var TEXTBOX:Int      = 9;
	public static inline var TEXTWORD:Int     = 10;
	public static inline var TOTAL_LAYERS:Int = 11;
	public static var displayLayers:Array<FlxGroup>;
	private static var mTilesets:Map < Int, String >; //class name to firstgid

	public static function init():Void
	{
		EventManager.subscribe(EventType.LEVEL_INIT, initLevel);
		EventManager.subscribe(EventType.LEVEL_DESTROY, destroyLevel);
	}
	
	private static function initLevel(evt:Int, params:Dynamic):Void
	{
		displayLayers = new Array<FlxGroup>();
		for (i in 0 ... MapLoader.TOTAL_LAYERS)
		{
			var group = new FlxGroup();
			displayLayers.push(group);
			FlxG.state.add(group);
		}
		
		mCollisionMap = new FlxTilemapExt();
//		displayLayers[0].add(mCollisionMap);
		
		loadMap(params.level);
	}
	
	private static function destroyLevel(evt:Int, params:Dynamic):Void
	{
		removeDisplayLayersFromScene();
	}
	
	private static function removeDisplayLayersFromScene():Void
	{
		for (i in 0 ... displayLayers.length)
		{
			FlxG.state.remove(displayLayers[i]);
			displayLayers[i].clear(); //removes references to any pool instances
		}
	}
	
	private static function setupTilesets(tilesetsData:Array<Dynamic>):Void
	{
		mTilesets = new Map < Int, String >();
		for (tileset in tilesetsData)
		{
			mTilesets.set(tileset.firstgid, tileset.name);
		}
	}
	
	private static function getClassTypeFromGid(gid:Int):String
	{
		var rettype:String = "";
		var storeClosestFirstGid:Int = -1;
		for (firstgid in mTilesets.keys())
		{
			if (gid == firstgid)
			{
				storeClosestFirstGid = firstgid;
				rettype = mTilesets[firstgid];
				break;
			}
			if (gid > firstgid)
			{
				if (storeClosestFirstGid == -1 || storeClosestFirstGid < firstgid)
					storeClosestFirstGid = firstgid;
			}
		}
		if (storeClosestFirstGid == -1)
			throw "error";
		if (rettype == "")
			rettype = mTilesets[storeClosestFirstGid];
		return rettype;
	}
	
	private static function loadMap(mapFile:String):Void
	{
		if (mapFile != "")
		{
			var obj = haxe.Json.parse(Assets.getText(mapFile));

			setupTilesets(obj.tilesets);
			var layers:Array<Dynamic> = obj.layers;
			for (layer in layers)
			{
				if (layer.name == "Terrain")
					readTileMap(layer);
				else if (layer.name == "BedIn")
					readObjects(layer, OBJECTLAYER0);
				else if (layer.name == "MainObjects")
					readObjects(layer, OBJECTLAYER1);
				else if (layer.name == "BedOut")
					readObjects(layer, OBJECTLAYER2);
				else if (layer.name == "Background")
					readBackground(layer);
			}
		}
	}
	
	private static function readTileMap(mapObject:Dynamic):Void
	{
		// fix up array
		var myarray:Array<Int> = mapObject.data;
		for (i in 0 ... myarray.length)
			myarray[i] -= myarray[i] != 0 ? 1 : 0;

		mCollisionMap.loadMap(FlxStringUtil.arrayToCSV(myarray, mapObject.width), Assets.getBitmapData("assets/img/terrain/tiles.png"), TILE_SIZE, TILE_SIZE);
		mCollisionMap.follow();
		mCollisionMap.immovable = true;
	}
	
	private static function readBackground(mapObject:Dynamic):Void
	{
		var path:String = mapObject.image;
		var parts:Array<String> = path.split("/");
		path = parts[parts.length - 1];
		var bg = new FlxSprite(0, 0, Assets.getBitmapData("assets/img/backgrounds/" + path));
		displayLayers[BACKGROUND].add(bg);
	}
	
	private static function readObjects(mapObject:Dynamic, displayLayerIndex:Int):Void
	{
		var objectsArray:Array<Dynamic> = mapObject.objects;
		for (obj in objectsArray)
		{
			var gameObj = GameObjectManager.getObjectFromPool(getClassTypeFromGid(obj.gid));
			gameObj.loadedName = obj.name;
			gameObj.setPosition(obj.x, obj.y - gameObj.frameHeight);
			gameObj.initEntity(displayLayers[displayLayerIndex]);
			
			//trace("obj: " + gameObj.frameHeight);
		}
	}
	
}
