package patients;
import event.EventManager;
import event.EventType;
import flixel.FlxG;
import objects.Fly;
import objects.MapLoader;
import openfl.Assets;
import flixel.FlxSprite;
import flixel.util.FlxArrayUtil;
import states.PlayState;

/**
 * ...
 * @author chinykian
 */
class Patient extends FlxSprite
{
	public static inline var CHILD:Int = 0;
	public static inline var TEEN:Int  = 1;
	public static inline var ADULT:Int = 2;
	public static inline var SENIOR:Int = 3;
	public static inline var NUM_AGE_GROUPS:Int = 4;
	
	public static inline var MALE:Int = 0;
	public static inline var FEMALE:Int = 1;

	public static var CONDITIONS:Array<String> = ["ill/injured", "urgent", "critical", "deceased"];
	
	public var ageFactor:Float;  //probability to critical condition
	public var ageGroup:Int = 0; 
	public var age:Int;
	public var gender:Int = 0;
	public var name:String;
	public var treatmentTime:Int;
	public var condition:String;
	public var quote:String;
	public var decayLevel:Int = 0;
	public var imagePath:String;
	
	private var mToFadeOut:Bool;

	public function new()
	{
		super();
		loadGraphic(Assets.getBitmapData("assets/img/characters/patients.png"), true, false, 32, 64);
		animation.add("empty", [0], 1, true);
		animation.add("male-adult", [1], 1, true);
		animation.add("male-teen", [2], 1, true);
		animation.add("male-child", [3], 1, true);
		animation.add("male-senior", [4], 1, true);
		animation.add("female-adult", [5], 1, true);
		animation.add("female-teen", [6], 1, true);
		animation.add("female-child", [7], 1, true);
		animation.add("female-senior", [8], 1, true);
		animation.play("empty");
	}
	
	public function isFadingOut():Bool
	{
		return mToFadeOut;
	}
	
	public function fadeOut():Void
	{
		decayLevel = 0;
		mToFadeOut = true;
		EventManager.triggerEvent(EventType.PATIENT_CURED, {patient:this});
	}
	
	public function generateDataAndSpawn():Void
	{
		color = 0xffffffff;
		alpha = 1;
		mToFadeOut = false;
		
		//generate gender
		gender = Math.random() < 0.5 ? MALE : FEMALE;
		//generate age group
		ageGroup = MathUtil.randomIntBetween(CHILD, NUM_AGE_GROUPS);
		switch (ageGroup)
		{
			case CHILD:
				age = MathUtil.randomIntBetween(3, 13);
				ageFactor = 0.9;
			case TEEN:
				age = MathUtil.randomIntBetween(13, 22);
				ageFactor = 0.7;
			case ADULT:
				age = MathUtil.randomIntBetween(22, 65);
				if (age <= 50)
					ageFactor = 0.5;
				else
					ageFactor = 0.7;
			case SENIOR:
				age = MathUtil.randomIntBetween(66, 91);
				ageFactor = 0.9;
		}
		
		quote = PatientLines.getQuoteByAge(age);
		decayLevel = 0;
		
		var genderString = gender == MALE ? "male" : "female";
		var ageGroupStrings = ["-child", "-teen", "-adult", "-senior"];
		var animString:String = genderString + ageGroupStrings[ageGroup];
		animation.play(animString);

		name = gender == MALE ? PatientNames.getMale() : PatientNames.getFemale();
		treatmentTime = getTreatmentTime();
		condition = CONDITIONS[0]; //TODO: gradually allow other possibilities
		imagePath = PatientProfileImages.getImageString(ageGroup, gender);
	}
	
	public override function update():Void
	{
		if (mToFadeOut)
		{
			alpha -= FlxG.elapsed;
			if (alpha <= 0)
			{
				alpha = 0;
				animation.play("empty");
			}
		}
		super.update();
	}
	
	public function advanceDecayLevel():Void
	{
		// update decay level
		if (condition == "deceased")
		{
			decayLevel++;
			if (decayLevel > 3)
				decayLevel = 3; //max cap
			return;
		}
	}

	public function advanceCondition():Void
	{
		var conditionIndex = CONDITIONS.indexOf(condition);
		if (conditionIndex < CONDITIONS.length-1)
		{
			//TODO: trigger event?
			condition = CONDITIONS[conditionIndex + 1];
			if (condition == "deceased")
			{
				EventManager.triggerEvent(EventType.PATIENT_DIED, null);
				color = 0xff444444;
			}
		}
	}

	private function getTreatmentTime():Int
	{
		var state:PlayState = cast FlxG.state;
		if (state.getTurn() <= 6)
		{
			var r = Math.random();
			if (r <= 0.7)
				return 1;
			else if (r <= 0.9)
				return 2;
			else
				return 3;
		}
		else
		{
			return MathUtil.randomIntBetween(1, 4);
		}
	}
	
}

class PatientNames
{
	public static var femaleNames:Array<String> = [
		'Joy Hack',
		'Shanta Gebhardt',
		'Shu Figgins',
		'Alida Runkle',
		'Tu Boehm',
		'Frederica Stallworth',
		'Catharine Lank',
		'Nella Kibby',
		'Vannesa Hann',
		'Arlena Dameron',
		'Cherri Crofoot',
		'Renita Serafin',
		'Maximina Wachowiak',
		'Lynelle Burkes',
		'Zonia Lemanski',
		'Reita Taveras',
		'Macie Carns',
		'Latasha Averitt',
		'Clara Yokley',
		'Narcisa Cooter',
		'Ciara Schroer',
		'Donita Chism',
		'Thu Santillan',
		'Bea Monson',
		'Caridad Corella',
		'Bula Eckhoff',
		'Ruby Helms',
		'Ayana Winchenbach',
		'Luann Keogh',
		'Tam Neira',
		'Glendora Tunstall',
		'Emmy Richert',
		'Juliane Holloway',
		'Lanelle Whitehill',
		'Sharee Hutson',
		'Ilse Moorehead',
		'Jeanice Burgener',
		'Carly Pomeroy',
		'Lean Custer',
		'Katherin Sedlock',
		'Kiera Schow',
		'Rosenda Greenan',
		'Natashia Peet',
		'Jennifer Wrench',
		'Dani Glantz',
		'Adrianna Schaar',
		'Terresa Hans',
		'Rosalia Valerio',
		'Tanja Koons',
		'Karisa Hulings'];

	public static var maleNames:Array<String> = [
		'Jarred Welliver',
		'Rocco Reinhard',
		'Deon Glisson',
		'Dick Condon',
		'Alva Rideout',
		'Denver Beeler',
		'Tim Wooten',
		'Raphael Aronson',
		'Wyatt Quattlebaum',
		'Jared Umana',
		'Rolf Mcmasters',
		'Roger Dahlin',
		'Foster Mooring',
		'Thaddeus Burriss',
		'Errol Corr',
		'Alvaro Cambron',
		'Edmond Wilfong',
		'Josh Branning',
		'Collin Scheff',
		'Abe Hawkin',
		'Damien Coghill',
		'Roosevelt Graden',
		'Basil Strobel',
		'Hilario Bury',
		'Minh Neary',
		'Irwin Hilt',
		'Terrell Blaylock',
		'Kent Resendes',
		'Raul Facemire',
		'Henry Mungia',
		'William Rigg',
		'Wilfredo Howlett',
		'Abdul Shreffler',
		'Renaldo Bender',
		'Erin Wylie',
		'Wesley Hendren',
		'Reid Motto',
		'Nestor Evelyn',
		'Tyree Jeffers',
		'Arron Bradeen',
		'Vernon Greenspan',
		'Pierre Ayoub',
		'Gerry Pittman',
		'Donny Mcquiston',
		'Virgilio Whalley',
		'Jimmie Talbott',
		'Dewayne Haskell',
		'Clair Lawlor',
		'Franklyn Caudillo',
		'Thomas Cruzan'];

	public static function getMale():String
	{
		return FlxArrayUtil.getRandom(maleNames);
	}

	public static function getFemale():String
	{
		return FlxArrayUtil.getRandom(femaleNames);
	}
}

class PatientProfileImages
{
	public static var Images = [
		Patient.ADULT => [
			Patient.FEMALE => [
				"profiles/adult_f_00.png",
				"profiles/adult_f_01.png",
				"profiles/adult_f_02.png",
				"profiles/adult_f_03.png",
				"profiles/adult_f_04.png",
				"profiles/adult_f_05.png" ],
			Patient.MALE => [
				"profiles/adult_m_00.png",
				"profiles/adult_m_01.png",
				"profiles/adult_m_02.png",
				"profiles/adult_m_03.png",
				"profiles/adult_m_04.png",
				"profiles/adult_m_05.png" ]
			],
		Patient.CHILD => [
			Patient.FEMALE => [
				"profiles/child_f_00.png",
				"profiles/child_f_01.png",
				"profiles/child_f_02.png",
				"profiles/child_f_03.png",
				"profiles/child_f_04.png",
				"profiles/child_f_05.png",
				"profiles/child_f_06.png" ],
			Patient.MALE => [
				"profiles/child_m_00.png",
				"profiles/child_m_01.png",
				"profiles/child_m_02.png",
				"profiles/child_m_03.png",
				"profiles/child_m_04.png",
				"profiles/child_m_05.png" ]
			],
		Patient.SENIOR => [
			Patient.FEMALE => [
				"profiles/senior_f_00.png",
				"profiles/senior_f_01.png",
				"profiles/senior_f_02.png" ],
			Patient.MALE => [
				"profiles/senior_m_00.png",
				"profiles/senior_m_01.png" ]
			],
		Patient.TEEN => [
			Patient.FEMALE => [
				"profiles/teen_f_00.png",
				"profiles/teen_f_01.png",
				"profiles/teen_f_02.png",
				"profiles/teen_f_03.png",
				"profiles/teen_f_04.png",
				"profiles/teen_f_05.png",
				"profiles/teen_f_06.png",
				"profiles/teen_f_07.png" ],
			Patient.MALE => [
				"profiles/teen_m_00.png",
				"profiles/teen_m_01.png",
				"profiles/teen_m_02.png",
				"profiles/teen_m_03.png",
				"profiles/teen_m_04.png",
				"profiles/teen_m_05.png" ],
			]
	];

	public static function getImageString(ageGroup:Int, gender:Int):String
	{
		return FlxArrayUtil.getRandom(Images[ageGroup][gender]);
	}
}
