package states;

import clock.TimeDisplay;
import flash.events.MouseEvent;
import flash.Lib;
import flixel.FlxG;
import flixel.effects.particles.FlxEmitterExt;
import flixel.FlxBasic;
import flixel.FlxCamera;
import flixel.FlxObject;
import flixel.FlxSprite;
import flixel.FlxState;
import flixel.group.FlxGroup;
import flixel.text.FlxText;
import flixel.util.FlxArrayUtil;
import flixel.util.FlxColor;
import flixel.util.FlxColorUtil;
import flixel.util.FlxPoint;
import fsm.FSM;
import fx.FxManager;
import hud.GameOverText;
import hud.InfectionDisplay;
import hud.TreatingAlertText;
import openfl.Assets;
import objects.*;
import objects.Monitor;
import objects.MapLoader;
import patients.Patient;
import tweenx909.TweenX;
import event.EventManager;
import event.EventType;
import hud.PatientInfoDisplay;
import hud.MiniInfoDisplay;

/**
* author: RottenMage
*/
class PlayState extends FlxStateExt
{
	private static inline var MAX_FADING_ALPHA = 0.5;
	
	private var mTurn:Int;
	private var mFSM:FSM;
	private var mPatientInfoDisplay:PatientInfoDisplay;
	private var mMiniInfoDisplay:MiniInfoDisplay;
	private var mTurnDisplay:FlxText;

	private var mInited:Bool;
	private var mHasTurnEnded:Bool;
	private var mIsProcessing:Bool;
	private var mNumberToSpawn:Int;
	
	private var mFadingBlackLayer:FlxSprite;
	private var mTimer:TimeDisplay;
	private var mTreatingDisplay:TreatingAlertText;
	private var mTurnWaitTime:Float;
	private var mInfectionDisplay:InfectionDisplay;
	
	public var mAreBedEnabled:Bool;
	public var mCurrentPatient:Patient;
	
	private var mCurrInfection:Float = 0;
	private var mDeathCount:Int = 0;
	
	private var mGameOverText:GameOverText;

	public function getTurn():Int
	{
		return mTurn;
	}

	override public function create():Void
	{
		super.create();
		FlxG.camera.bgColor = 0xff000000;
		mFadingBlackLayer = new FlxSprite();
		mFadingBlackLayer.makeGraphic(FlxG.width, FlxG.height, FlxColor.BLACK);
		mFadingBlackLayer.scrollFactor.set(0, 0);
		mFadingBlackLayer.alpha = 0;

		EventManager.init();
		MapLoader.init();
		GameObjectManager.init();

		BedObject.allBeds = new Array<BedObject>();
		Monitor.AllMonitors = new Array<Monitor>();

		EventManager.triggerEvent(EventType.LEVEL_INIT, {level:"assets/levels/hospital.json"});
		EventManager.update();

		mTurn = 0;
		mInited = false;
		mHasTurnEnded = false;
		mIsProcessing = false;
		mAreBedEnabled = true;
		mNumberToSpawn = 5;
		
		mCurrentPatient = null;

		mFSM = new FSM();
		mFSM.addState("init", [initStateEnter, initStateLoop, initStateExit],
					  ["basic" => inittobasic]);
		mFSM.addState("basic", [basicStateEnter, basicStateLoop, basicStateExit],
					  ["process" => basictoprocess,
					   "gameover" => basictogameover]);
		mFSM.addState("process", [processStateEnter, processStateLoop, processStateExit],
					  ["basic" => processtobasic]);
		mFSM.addState("turn_anim", [turnAnimStateEnter, turnAnimStateLoop, turnAnimStateExit],
					  ["basic" => turnanimtobasic,
					   "process" => turnanimtoprocess]);
		mFSM.addState("gameover", [gameoverStateEnter, gameoverStateLoop, gameoverStateExit],
					  null);
					   
		MapLoader.displayLayers[MapLoader.OVERLAY].add(mFadingBlackLayer);
		
		mMiniInfoDisplay = new MiniInfoDisplay();
		mMiniInfoDisplay.initEntity(MapLoader.displayLayers[MapLoader.TEXTBOX]);
		mPatientInfoDisplay = new PatientInfoDisplay();
		mPatientInfoDisplay.initEntity(MapLoader.displayLayers[MapLoader.TEXTBOX]);
		
		mTimer = new TimeDisplay();
		mTimer.initEntity(MapLoader.displayLayers[MapLoader.OVERLAY]);
		
		mTreatingDisplay = new TreatingAlertText();
		mTreatingDisplay.initEntity(MapLoader.displayLayers[MapLoader.OVERLAY]);
		mTreatingDisplay.visible = false;
		
		mInfectionDisplay = new InfectionDisplay();
		mInfectionDisplay.initEntity(MapLoader.displayLayers[MapLoader.OVERLAY]);
		
		EventManager.subscribe(EventType.DISPLAY_INFO, onDisplayInfo);
		EventManager.subscribe(EventType.DISPLAY_CANCEL, onDisplayCancel);
		EventManager.subscribe(EventType.TREAT_PATIENT, onTreatPatient);
		EventManager.subscribe(EventType.CLEAR_PATIENT, onTreatPatient);
		EventManager.subscribe(EventType.PATIENT_DIED, onPatientDied);
		EventManager.subscribe(EventType.TURN_PASSED, onTurnPassed);
		EventManager.subscribe(EventType.TURN_PASSED_OVER, onTurnPassedOver);

		for (bed in BedObject.allBeds)
		{
			bed.onMouseOver = onMouseOverBed;
			bed.onMouseOut = onMouseOutBed;
		}

		mGameOverText = new GameOverText();
		mGameOverText.initEntity(MapLoader.displayLayers[MapLoader.OVERLAY]);
		AudioMixer.enterState(this);
		
		FlxG.camera.fade(0xff000000, 1, true);
	}
	
	override public function update():Void
	{
		EventManager.update();
		mInited = true;

		super.update();
		mFSM.update();
		AudioMixer.update();
		FxManager.update();
	}
	
	public function getInfectionLevel():Float
	{
		var count:Int = 0;
		for (bed in BedObject.allBeds)
		{
			if (bed.isOccupied())
			{
				count += bed.patient.decayLevel;
			}
		}
		
		var maxInfectionCount:Int = 12;
		if (count >= maxInfectionCount)
			return 1;
		return cast(count, Float) / maxInfectionCount;
	}
	
	public override function onMouseMove(evt:MouseEvent):Void
	{
		super.onMouseMove(evt);
	}

	private function onDisplayInfo(evt:Int, params:Dynamic):Void
	{
		mMiniInfoDisplay.hide();

		var patient:Patient = params.patient;
		mPatientInfoDisplay.showInfo(patient, new FlxPoint(0,0));

		mPatientInfoDisplay.x = (FlxG.width - mPatientInfoDisplay.width) / 2;
		mPatientInfoDisplay.y = (FlxG.height - mPatientInfoDisplay.height) / 2;
		
		mCurrentPatient = patient;

		AudioMixer.playHeartbeatLouder(patient.condition);
	}
	private function onTreatPatient(evt:Int, params:Dynamic):Void
	{
		mHasTurnEnded = true;
		AudioMixer.stopHeartbeat();
	}
	private function onPatientDied(evt:Int, params:Dynamic):Void
	{
		mDeathCount++;
	}
	private function onTurnPassed(evt:Int, params:Dynamic):Void
	{
		mFSM.changeState("turn_anim");
	}
	private function onTurnPassedOver(evt:Int, params:Dynamic):Void
	{
		updateInfectionLevel();
	}
	private function isFree(bed:BedObject):Bool
	{
		return !bed.isOccupied();
	}

	private function onDisplayCancel(evt:Int, params:Dynamic):Void
	{
		AudioMixer.stopHeartbeat();
	}


	// States
	private function initStateEnter():Void
	{
	}
	private function initStateLoop():Void
	{
	}
	private function initStateExit():Void
	{
		//FlxArrayUtil.getRandom(BedObject.allBeds).spawnPatient();
	}

	private function basicStateEnter():Void
	{
		mHasTurnEnded = false;
		mTurn += 1;
		
		while (mNumberToSpawn > 0)
		{
			var freeBeds:Array<BedObject> = Lambda.array(Lambda.filter(BedObject.allBeds, isFree));
			if (freeBeds.length > 0)
			{
				FlxArrayUtil.getRandom(freeBeds).spawnPatient();
			}
			mNumberToSpawn -= 1;
		}
	}
	private function basicStateLoop():Void
	{
		mTreatingDisplay.visible = false;
		mFadingBlackLayer.alpha -= FlxG.elapsed;
		mFadingBlackLayer.alpha = Math.max(0, mFadingBlackLayer.alpha);
	}
	private function basicStateExit():Void
	{
	}

	private function processStateEnter():Void
	{
		mIsProcessing = true;
		mAreBedEnabled = false;
		EventManager.triggerEvent(EventType.TURN_PASSED, {patient:mCurrentPatient});
		
		mTreatingDisplay.visible = true;
		mTreatingDisplay.setPosition(mCurrentPatient.x - 35, mCurrentPatient.y - 15);
		
		if (mCurrentPatient.condition == "deceased")
		{
			mCurrentPatient.treatmentTime = 1;
			mTreatingDisplay.setText("CLEARING BODY");
		}
		else
		{
			mTreatingDisplay.setText("TREATING");
		}

		AudioMixer.startTicking();
	}
	private function processStateLoop():Void
	{
		mIsProcessing = false;
	}
	private function processStateExit():Void
	{
		mHasTurnEnded = false;
	}
	
	private function turnAnimStateEnter():Void
	{
		mCurrentPatient.treatmentTime--;
		mNumberToSpawn++;
		mTurnWaitTime = 0;
	}
	private function turnAnimStateLoop():Void
	{
		mFadingBlackLayer.alpha += FlxG.elapsed * 0.4;
		mTurnWaitTime += FlxG.elapsed * 0.4;
		mFadingBlackLayer.alpha = Math.min(MAX_FADING_ALPHA, mFadingBlackLayer.alpha);
	}
	private function turnAnimStateExit():Void
	{
		// Update the status of each bed
		for (bed in BedObject.allBeds)
		{
			if (bed.patient != mCurrentPatient)
				bed.updateStatus();
		}
	
		EventManager.triggerEvent(EventType.TURN_PASSED_OVER, null);
		if (mCurrentPatient.treatmentTime <= 0)
		{
			mCurrentPatient.fadeOut();
			mAreBedEnabled = true;
			AudioMixer.stopTicking();
		}
		
		//updateInfectionLevel();
	}
	
	private var gameOverStep:Int = 0;
	private function gameoverStateEnter():Void
	{
		mInfectionDisplay.visible = false;
		mTimer.visible = false;
		gameOverStep = 0;
		
		mGameOverText.start(mDeathCount);
	}
	private function gameoverStateLoop():Void
	{
		gameOverStep++;
		gameOverStep = gameOverStep > 100 ? 100 : gameOverStep;
		FlxG.camera.color = FlxColorUtil.interpolateColor(0xffffff, 0xff1111, 100, gameOverStep, 255);
	}
	private function gameoverStateExit():Void
	{
	}
	
	private function updateInfectionLevel():Void
	{
		var oldInfectionLevel = mCurrInfection;
		
		mCurrInfection = getInfectionLevel();
		mInfectionDisplay.setPercentage(mCurrInfection);
		
		if (mCurrInfection >= 0.6 && oldInfectionLevel < 0.6)
			mInfectionDisplay.startBlinking();
		else if (mCurrInfection < 0.6 && oldInfectionLevel >= 0.6)
			mInfectionDisplay.stopBlinking();
	}

	// Edges
	private function inittobasic():Bool
	{
		return mInited;
	}

	private function basictoprocess():Bool
	{
		return mHasTurnEnded;
	}
	
	private function basictogameover():Bool
	{
		return mCurrInfection >= 1;
	}

	private function processtobasic():Bool
	{
		return !mIsProcessing;
	}
	
	private function turnanimtobasic():Bool
	{
		return mCurrentPatient.treatmentTime <= 0 && mTurnWaitTime >= MAX_FADING_ALPHA;
	}
	
	private function turnanimtoprocess():Bool
	{
		return !turnanimtobasic() && mTurnWaitTime >= MAX_FADING_ALPHA;
	}

	// Helpers
	private function onMouseOverBed(bed:BedObject):Void
	{
		if (bed.patient.treatmentTime > 0)
		{
			mMiniInfoDisplay.showInfo(bed.patient, new FlxPoint(bed.centerX, bed.centerY));
			if (FlxG.width - (mMiniInfoDisplay.x + mMiniInfoDisplay.width) < 10)
			{
				mMiniInfoDisplay.x -= 10;
			}

			//HACK: don't play new heartbeat when the patient info display is shown
			if (mPatientInfoDisplay.x > FlxG.width)
				AudioMixer.playHeartbeat(bed.patient.condition, 0.5);
		}
	}
	private function onMouseOutBed(bed:BedObject):Void
	{
		mMiniInfoDisplay.hide();
		//HACK: don't stop heartbeat when the patient info display is shown
		if (mPatientInfoDisplay.x > FlxG.width)
			AudioMixer.stopHeartbeat();
	}
}
